local SLASH_TEXTURE = Resources.load_texture("spell_sword_slashes.png")
local BLADE_TEXTURE = Resources.load_texture("elem_sword_blades.png")
local AUDIO = Resources.load_audio("sfx.ogg")

function card_mutate(player, index)
	local card = player:field_card(index);

	local list = player:field_cards()
	local length = #list

	local i = index + 1
	local next_card;

	while i < length do
		next_card = player:field_card(i)
		if next_card and next_card.damage > 0 then
			card.damage = card.damage + next_card.damage
			card.boosted_damage = card.boosted_damage + next_card.damage
			player:remove_field_card(i)
		else
			i = i + 1
		end
	end
	player:set_field_card(index, card)
end

function card_init(actor, props)
	local action = Action.new(actor, "CHARACTER_SWING")
	action:set_lockout(ActionLockout.new_animation())

	action.on_execute_func = function(self, user)
		self:add_anim_action(2, function()
			local hilt = self:create_attachment("HILT")
			local hilt_sprite = hilt:sprite()
			hilt_sprite:set_texture(actor:texture())
			hilt_sprite:set_layer(-2)
			hilt_sprite:use_root_shader(true)

			local hilt_anim = hilt:animation()
			hilt_anim:copy_from(actor:animation())
			hilt_anim:set_state("HILT")

			local blade = hilt:create_attachment("ENDPOINT")
			local blade_sprite = blade:sprite()
			blade_sprite:set_texture(BLADE_TEXTURE)
			blade_sprite:set_layer(-1)

			local blade_anim = blade:animation()
			blade_anim:load("elem_sword_blades.animation")
			blade_anim:set_state("WOOD")
		end)

		self:add_anim_action(3, function()
			local sword = create_slash(user, props)
			local tile = user:get_tile(user:facing(), 1)

			local sharebox1 = SharedHitbox.new(sword, 9)
			local sharebox2 = SharedHitbox.new(sword, 9)

			sharebox1:set_hit_props(sword:copy_hit_props())
			sharebox2:set_hit_props(sword:copy_hit_props())

			actor:field():spawn(sharebox1, tile:get_tile(Direction.Up, 1))
			actor:field():spawn(sword, tile)
			actor:field():spawn(sharebox2, tile:get_tile(Direction.Down, 1))

			local fx = Artifact.new()
			fx:set_facing(sword:facing())
			fx:set_texture(SLASH_TEXTURE, true)

			local anim = fx:animation()

			anim:load("spell_sword_slashes.animation")
			anim:set_state("WIDE")
			anim:on_complete(function()
				fx:erase()
				sword:erase()
			end)

			actor:field():spawn(fx, tile)
		end)
	end
	return action
end

function create_slash(user, props)
	local spell = Spell.new(user:team())
	spell:set_facing(user:facing())
	spell:set_tile_highlight(Highlight.Flash)
	spell:set_hit_props(
		HitProps.new(
			props.damage,
			props.hit_flags,
			props.element,
			props.secondary_element,
			user:context(),
			Drag.None
		)
	)
	spell.on_update_func = function(self)
		local current_tile = self:current_tile();
		local up_tile = current_tile:get_tile(Direction.Up, 1)
		local down_tile = current_tile:get_tile(Direction.Down, 1)

		if up_tile and not up_tile:is_edge() then
			up_tile:set_highlight(Highlight.Flash)
		end

		if down_tile and not down_tile:is_edge() then
			down_tile:set_highlight(Highlight.Flash)
		end
		current_tile:attack_entities(self)
	end

	spell.can_move_to_func = function(tile)
		return true
	end

	spell.on_collision_func = function(self, other)
		self:erase()
	end

	Resources.play_audio(AUDIO)

	return spell
end
